/*jslint newcap: true, plusplus: true, nomen: true */
/*global $, Alloy, L, Widget, _, OS_IOS */

(function (args) {
    "use strict";
    
    var moment = require("alloy/moment"),
    
        // arguments
        name = args.name,
        events = args.events;
    
    function getHourMinute(time, isStartTime) {
        
        var result,
            timeMoment;
        
        if (!time) {
            if (isStartTime) {
                result = "0000";
            } else {
                result = "2359";
            }
        } else {
            timeMoment = moment.utc(time);
            result = timeMoment.format("HHmm");
        }
        
        return result;
    }
    
    function compareEvents(event1, event2) {
        // Comparing dates using > or < returns the expected value, but using == or === always returns 0.  Therefore
        // use only > and < when comparing dates here.
        
        var startDate1,
            startDate2,
            endDate1,
            endDate2,
            startTime1,
            startTime2,
            endTime1,
            endTime2;
        
        //Start date should always be set appropriately.    
        startDate1 = event1.startDate;
        startDate2 = event2.startDate;
        
        //If there is no end date, event is assumed to end at midnight the same day.
        endDate1 = event1.endDate;
        if (!endDate1) {
            endDate1 = startDate1;
        }
        endDate2 = event2.endDate;
        if (!endDate2) {
            endDate2 = startDate2;
        }
        
        //Ignore the date part of the time pieces, they are irrelevant.
        //If there is no start time, the event is assumed to start at midnight the previous night.
        startTime1 = getHourMinute(event1.startTime, true);
        startTime2 = getHourMinute(event2.startTime, true);
        
        //Ignore the date part of the time pieces, they are irrelevant.
        //If there is no end time, the event is assumed to end at midnight.
        endTime1 = getHourMinute(event1.endTime, false);
        endTime2 = getHourMinute(event2.endTime, false);
        
        if (startDate1 > startDate2) {
            return 1;
        }

        if (startDate1 < startDate2) {
            return -1;
        }

        if (startTime1 > startTime2) {
            return 1;
        }

        if (startTime1 < startTime2) {
            return -1;
        }
        
        if (endDate1 > endDate2) {
            return 1;
        }

        if (endDate1 < endDate2) {
            return -1;
        }

        if (endTime1 > endTime2) {
            return 1;
        }

        if (endTime1 < endTime2) {
            return -1;
        }

        // Start date/time and end date/times are the same, sort by name.
        return event1.name.localeCompare(event2.name);

    }

        /**
         * Format a date and time pair.
         * 
         * @param {Moment} [dateToFormat]
         * @param {Moment} [timeToFormat]
         * 
         * @return {String}
         * A string representation of the date/time pair, or empty string if
         * they are null.
         * i.e. "Sep 2", "Sep 2, 2:00 PM"
         */
        function formatDateAndTime (dateToFormat, timeToFormat) {

            var result,
                utcDate,
                utcTime;

            if (dateToFormat) {

                utcDate = moment.utc([dateToFormat.year(), dateToFormat.month(), dateToFormat.date()]);

                if (timeToFormat) {
                    //Sep 21, 5:00 PM
                    
                    utcTime = moment.utc([2015, 0, 1, timeToFormat.hour(), timeToFormat.minute()]);

                    utcDate.set('hour', utcTime.hour());
                    utcDate.set('minute', utcTime.minute());

                    result = utcDate.format("MMM D, h:mm A");

                } else {
                    //Sep 21
                    result = utcDate.format("MMM D");
                }

            } else {
                result = "";
            }

            return result;

        }

    /**
     * Format an event date for display.
     * 
     * @param {String} startDate
     * @param {String} [startTime]
     * @param {String} [endDate]
     * @param {String} [endTime]
     */
    function formatEventDate (startDate, startTime, endDate, endTime) {

        var dateText,
            startDateText,
            endDateText,
            hasStartDate = false,
            startEndDatesAreSame = false,
            hasStartTime = false,
            hasEndTime = false;

        if (startDate) {
            hasStartDate = true;
            startDate = moment.utc(startDate);
        }
        if (startTime) {
            hasStartTime = true;
            startTime = moment.utc(startTime);
        }
        if (endDate) {
            endDate = moment.utc(endDate);
        }
        if (hasStartDate && endDate) {
            startEndDatesAreSame = (endDate.format() === startDate.format());
        } else {
            startEndDatesAreSame = true;
        }
        if (endTime) {
            endTime = moment.utc(endTime);
            hasEndTime = true;
            
            // WI 529792
            // If the start and end time is the same and the end date isn't
            // different from the start date, don't display the end time.
            if (startEndDatesAreSame &&
                hasStartTime &&
                startTime.hour() === endTime.hour() &&
                startTime.minute() === endTime.minute()) {
                
                hasEndTime = false;
            }
            
        }

        if (hasStartDate) {

            if (startEndDatesAreSame) {

                if (hasEndTime) {
                    if (hasStartTime) {
                        //Sep 21, 5:00 PM – 8:00 PM
                        dateText = String.format("%s – %s",
                                                 formatDateAndTime(startDate, startTime),
                                                 endTime.format("h:mm A"));
                    } else {
                        //Sep 21, ends at 9:00 PM
                        dateText = String.format("%s, ends at %s",
                                                 startDate.format("MMM D"),
                                                 endTime.format("h:mm A"));
                    }
                } else {
                    dateText = formatDateAndTime(startDate, startTime);
                }

            } else {

                //Sep 21 – Sep 24
                //Sep 21 – Sep 24, 9:00 PM
                //Sep 21, 8:00 PM – Sep 24
                //Sep 21, 9:00 PM – Sep 22, 2:00 AM

                startDateText = formatDateAndTime(startDate, startTime);
                endDateText = formatDateAndTime(endDate, endTime);
                dateText = String.format("%s – %s", startDateText, endDateText);

            }

        } else {
            dateText = "";
        }

        return dateText;

    }

    function getListItem(event) {

        var props = {
            itemId: event.eventId.value
        };
       
        if (OS_IOS) {
            props = _.extend(props, {
                selectionStyle: Ti.UI.iPhone.ListViewCellSelectionStyle.NONE
            });
        }
        
        return {
            properties: props,
            nameLabel: _.extend({
                text: event.name
            })
        };
    }
    
    function getListItems(events) {
        
        var data = [];
        
        _.each(events, function (event) {
            data.push(getListItem(event));
        });
        
        return data;

    }
    
    function init() {
        
        $.sectionName.setText(name);
        
        events.sort(compareEvents);
        
        $.section.setItems(getListItems(events));

    }
    
    init();
    
}(arguments[0] || {}));